@@ -1,4 +1,4 @@
-/*  Copyright (C) 2020 CZ.NIC, z.s.p.o. <knot-dns@labs.nic.cz>
+/*  Copyright (C) 2021 CZ.NIC, z.s.p.o. <knot-dns@labs.nic.cz>
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -36,7 +36,7 @@
 /*** Query module API. ***/
 
 /*! Current module ABI version. */
-#define KNOTD_MOD_ABI_VERSION	200
+#define KNOTD_MOD_ABI_VERSION	300
 /*! Module configuration name prefix. */
 #define KNOTD_MOD_NAME_PREFIX	"mod-"
 
@@ -220,31 +220,37 @@ int knotd_mod_stats_add(knotd_mod_t *mod, const char *ctr_name, uint32_t idx_cou
  * Increments a statistics counter.
  *
  * \param[in] mod     Module context.
+ * \param[in] thr_id  Index of worker thread.
  * \param[in] ctr_id  Counter id (counted in the order the counters were registered).
  * \param[in] idx     Subcounter index (set 0 for single-counter).
  * \param[in] val     Value increment.
  */
-void knotd_mod_stats_incr(knotd_mod_t *mod, uint32_t ctr_id, uint32_t idx, uint64_t val);
+void knotd_mod_stats_incr(knotd_mod_t *mod, unsigned thr_id, uint32_t ctr_id,
+                          uint32_t idx, uint64_t val);
 
 /*!
  * Decrements a statistics counter.
  *
  * \param[in] mod     Module context.
+ * \param[in] thr_id  Index of worker thread.
  * \param[in] ctr_id  Counter id (counted in the order the counters were registered).
  * \param[in] idx     Subcounter index (set 0 for single-counter).
  * \param[in] val     Value decrement.
  */
-void knotd_mod_stats_decr(knotd_mod_t *mod, uint32_t ctr_id, uint32_t idx, uint64_t val);
+void knotd_mod_stats_decr(knotd_mod_t *mod, unsigned thr_id, uint32_t ctr_id,
+                          uint32_t idx, uint64_t val);
 
 /*!
  * Sets a statistics counter value.
  *
  * \param[in] mod     Module context.
+ * \param[in] thr_id  Index of worker thread.
  * \param[in] ctr_id  Counter id (counted in the order the counters were registered).
  * \param[in] idx     Subcounter index (set 0 for single-counter).
  * \param[in] val     Value.
  */
-void knotd_mod_stats_store(knotd_mod_t *mod, uint32_t ctr_id, uint32_t idx, uint64_t val);
+void knotd_mod_stats_store(knotd_mod_t *mod, unsigned thr_id, uint32_t ctr_id,
+                           uint32_t idx, uint64_t val);
 
 /*! Configuration single-value abstraction. */
 typedef union {
@@ -277,6 +283,7 @@ typedef enum {
 	KNOTD_CONF_ENV_HOSTNAME    = 1, /*!< Current hostname. */
 	KNOTD_CONF_ENV_WORKERS_UDP = 2, /*!< Current number of UDP workers. */
 	KNOTD_CONF_ENV_WORKERS_TCP = 3, /*!< Current number of TCP workers. */
+	KNOTD_CONF_ENV_WORKERS_XDP = 4, /*!< Current number of UDP-over-XDP workers. */
 } knotd_conf_env_t;
 
 /*!
@@ -302,6 +309,15 @@ knotd_conf_t knotd_conf(knotd_mod_t *mod, const yp_name_t *section_name,
  */
 knotd_conf_t knotd_conf_env(knotd_mod_t *mod, knotd_conf_env_t env);
 
+/*!
+ * Gets number of answering threads.
+ *
+ * \param[in] mod  Module context.
+ *
+ * \return Number of worker threads.
+ */
+unsigned knotd_mod_threads(knotd_mod_t *mod);
+
 /*!
  * Gets module configuration value.
  *
@@ -376,9 +392,8 @@ typedef enum {
 typedef enum {
 	KNOTD_QUERY_FLAG_NO_AXFR    = 1 << 0, /*!< Don't process AXFR. */
 	KNOTD_QUERY_FLAG_NO_IXFR    = 1 << 1, /*!< Don't process IXFR. */
-	KNOTD_QUERY_FLAG_LIMIT_ANY  = 1 << 2, /*!< Limit ANY QTYPE (respond with TC=1). */
-	KNOTD_QUERY_FLAG_LIMIT_SIZE = 1 << 3, /*!< Apply UDP size limit. */
-	KNOTD_QUERY_FLAG_COOKIE     = 1 << 4, /*!< Valid DNS Cookie indication. */
+	KNOTD_QUERY_FLAG_LIMIT_SIZE = 1 << 2, /*!< Apply UDP size limit. */
+	KNOTD_QUERY_FLAG_COOKIE     = 1 << 3, /*!< Valid DNS Cookie indication. */
 } knotd_query_flag_t;
 
 /*! Query processing data context parameters. */
@@ -388,6 +403,7 @@ typedef struct {
 	int socket;                            /*!< Current network socket. */
 	unsigned thread_id;                    /*!< Current thread id. */
 	void *server;                          /*!< Server object private item. */
+	struct knot_xdp_msg *xdp_msg;          /*!< Possible XDP message context. */
 } knotd_qdata_params_t;
 
 /*! Query processing data context. */
@@ -409,6 +425,26 @@ typedef struct {
 	struct knotd_qdata_extra *extra; /*!< Private items (process_query.h). */
 } knotd_qdata_t;
 
+/*!
+ * Gets the local (destination) address of the query.
+ *
+ * \param[in] qdata  Query data.
+ * \param[out] buff  Auxiliary buffer (not used for XDP).
+ *
+ * \return Local address or NULL if error.
+ */
+const struct sockaddr_storage *knotd_qdata_local_addr(knotd_qdata_t *qdata,
+                                                      struct sockaddr_storage *buff);
+
+/*!
+ * Gets the remote (source) address of the query.
+ *
+ * \param[in] qdata  Query data.
+ *
+ * \return Remote address or NULL if error.
+ */
+const struct sockaddr_storage *knotd_qdata_remote_addr(knotd_qdata_t *qdata);
+
 /*!
  * Gets the current zone name.
  *
